-------------------------------------------------------------------------------
-- |> SPLASH SCREEN
--: Creates a splash screen window for a static display of info
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local CreateFrame = CreateFrame
local UIFrameFadeOut = UIFrameFadeOut
-------------------------------------------------------------------------------
-- |> SETTINGS
-------------------------------------------------------------------------------
local SplashShownMinVersion = 1730 -- Show splash the\ this version number or less
local bgTexture = "Interface\\Addons\\weizPVP\\Media\\Textures\\white16x16.tga"
local edgeFile = "Interface\\Addons\\weizPVP\\Media\\Textures\\white16x16.tga"
local fontBold = "Interface\\Addons\\weizPVP\\Media\\Fonts\\RobotoCondensed-Bold.ttf"
local fontBoldItalic = "Interface\\Addons\\weizPVP\\Media\\Fonts\\RobotoCondensed-BoldItalic.ttf"

-------------------------------------------------------------------------------
--|> CREATE SPLASH SCREEN
-------------------------------------------------------------------------------
weizPVP.splash = {}
local SplashDetailsText = "|cff37ff37 New Features|r\n"
SplashDetailsText = SplashDetailsText .. "    • Phasing Notifications (Still in development)\n"
SplashDetailsText =
    SplashDetailsText .. "    • Enemies on Flight Paths are detected and given a FP icon on their player bar\n"
SplashDetailsText = SplashDetailsText .. "    • Recreated higher resolution image assets for sharper scaling.\n"
SplashDetailsText =
    SplashDetailsText .. "    • Some options cleaned up a little, and new options added for new features.\n"
SplashDetailsText = SplashDetailsText .. "    • Minimap icon has been added\n"
SplashDetailsText =
    SplashDetailsText .. "    • weizPVP can now be used in LibDataBroker displays like Bazooka or Elvui datatexts\n"
SplashDetailsText = SplashDetailsText .. "\n"

-- [[ Splash screen ]]

local splash = CreateFrame("Frame", "RecentChangesScreen", _G.UIParent)
splash:SetPoint("CENTER")
splash:SetMovable(true)
splash:EnableMouse(true)
splash:RegisterForDrag("LeftButton")
splash:SetScript("OnDragStart", splash.StartMoving)
splash:SetScript("OnDragStop", splash.StopMovingOrSizing)
splash:SetSize(520, 300)
splash:SetFrameStrata("DIALOG")
splash:SetBackdrop(
    {
        bgFile = bgTexture,
        tile = false,
        edgeFile = edgeFile,
        edgeSize = 1,
        insets = {left = 1, right = 1, top = 1, bottom = 1}
    }
)
splash:SetBackdropColor(0.1, 0.1, 0.1, 0.1)
splash:SetBackdropBorderColor(0, 0, 0, 1)
splash:Hide()

splash.bg = splash:CreateTexture(nil, "BACKGROUND")
splash.bg:SetPoint("TOPLEFT", splash, "TOPLEFT")
splash.bg:SetPoint("BOTTOMRIGHT", splash, "BOTTOMRIGHT")
splash.bg:SetDrawLayer("BACKGROUND", 0)
splash.bg:SetColorTexture(1, 1, 1, 1)
splash.bg:SetGradientAlpha("VERTICAL", 0.1, 0.1, 0.1, 0.7, 0.08, 0.08, 0.08, 0.8)

do
    local text =
        [[
Thank you for using weizPVP!

If you come accross any bugs please be sure to report them! It helps me a ton!
Go to the weizPVP's project page on curseforge, and click |cff37ff37'Issues'|r at the top.

]] ..
        SplashDetailsText .. [[

Happy Hunting!
]]
    local title = splash:CreateFontString(nil, "ARTWORK", "GameFont_Gigantic")
    title:SetFont(fontBold, 26, "NONE")
    title:SetPoint("TOP", 0, -25)
    title:SetHeight(26)
    title:SetText("|cffffffffweiz|r|cffffa012PVP|r |cFFFFFFFF:|r |cFFaaaaaaRecent Changes|r")

    -- --> VERSION
    local version = splash:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    version:SetPoint("BOTTOMRIGHT", splash, "BOTTOMRIGHT", -8, 8)
    version:SetJustifyH("RIGHT")
    version:SetFont(fontBoldItalic, 12, "NONE")
    version:SetText("|cFFbbbbcc" .. _G.GetAddOnMetadata("weizPVP", "Version") .. "|r")
    version:SetShadowColor(0, 0, 0, 0)

    local body = splash:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    body:SetPoint("TOP", title, "BOTTOM", 0, -20)
    body:SetWidth(splash:GetWidth() - 34)
    body:SetJustifyH("LEFT")
    body:SetText(text)

    local okayButton = CreateFrame("Button", nil, splash, "UIPanelButtonTemplate")
    okayButton:SetSize(128, 25)
    okayButton:SetPoint("BOTTOM", 0, 10)
    okayButton:SetText("Got it")
    okayButton:SetScript(
        "OnClick",
        function()
            splash:Hide()
            --> CLEAN ALL SETTINGS
            --> Fade out
            UIFrameFadeOut(splash, 0.3, 1, 0)
            weizPVP.Options.Addon.VersionSplashSeen = true
            splash:SetMovable(false)
            splash:EnableMouse(false)
            splash.okayButton:EnableMouse(false)
        end
    )

    splash.okayButton = okayButton
    splash.closeButton = CreateFrame("Button", nil, splash, "UIPanelCloseButton")
    local NewHeight = body:GetHeight() + okayButton:GetHeight() + title:GetWidth()
    splash:SetHeight(NewHeight * 0.71)
end

function weizPVP:CreateSplashScreen()
    --> CREATE SPLASH WINDOW
    weizPVP.Splash = splash
end

-------------------------------------------------------------------------------
--> Check Splash
-------------------------------------------------------------------------------
function weizPVP:CheckSplash()
    --> Reset if not seen splash screen before
    if not weizPVP.Options.Addon.VersionSplashSeen or weizPVP.Options.Addon.VersionSplashSeen == false then
        --> LOAD CHARACTER OPTIONS
        weizPVP.charDB:ResetDB()
        weizPVP.globalDB:ResetDB()
        weizPVP.charDB = LibStub("AceDB-3.0"):New("weizPVPOptions", weizPVP.defaults, true)
        weizPVP.Options = weizPVP.charDB.profile.Options
        --> LOAD GLOBAL PLAYER DB
        weizPVP.globalDB = LibStub("AceDB-3.0"):New("weizPVPDatabase", {}, false)
        weizPVP.PlayerDB = weizPVP.globalDB.global
    end

    --> Check to see if new version popup has been seen for this version
    local ShowSplash = not weizPVP.Options.Addon.VersionNumber or false

    ShowSplash = ShowSplash or weizPVP.Options.Addon.VersionNumber < SplashShownMinVersion

    ShowSplash = ShowSplash or weizPVP.Options.DEBUG == true

    if ShowSplash then
        weizPVP.Splash:Show()
    end

    --> Update Versions
    weizPVP.Options.Addon.Version = weizPVP.Addon.VERSION
    weizPVP.Options.Addon.VersionNumber = weizPVP.Addon.VERSION_NUM
end
